/*#############################################################################
  # Copyright (c) 2010 Giovanni "Raptor" Pessiva <giovanni.pessiva@gmail.com> #
  # This program is free software: you can redistribute it and/or modify      #
  # it under the terms of the GNU General Public License version 3            #
  # as published by the Free Software Foundation.                             #
  #############################################################################*/
#include "newname.h"
#include "parsedfile.h"
#include "preview.h"
#include <QDebug>
#include <QBoxLayout>
#include <QLabel>
#include <QMessageBox>
#include <QDialog>
#include <QFileDialog>

NewName::NewName(QFileInfo *fileInfo, QWidget *parent) :
    QDialog(parent),
    m_sample(new ParsedFile(fileInfo->absoluteFilePath())),
    m_directory(new QDir(fileInfo->canonicalPath()))
{
    setFilesList();

    setLayout(new QBoxLayout(QBoxLayout::TopToBottom));

    layout()->addWidget(new QLabel(tr("Vecchio nome: <i>\"")+m_sample->toAscii()+"\"</i>"));
    layout()->addWidget(new QLabel(tr("Inserisci un nuovo nome:")));
    m_newName=new QLineEdit(m_sample->toAscii());
    layout()->addWidget(m_newName);


    m_buttons=new QBoxLayout(QBoxLayout::LeftToRight);
    m_ok=new QPushButton(tr("Ok"));
    m_ok->setDefault(true);
    m_cancel=new QPushButton(tr("Annulla"));
    m_buttons->addWidget(m_ok);
    m_buttons->addWidget(m_cancel);
    ((QBoxLayout *) layout())->addLayout(m_buttons);

    QObject::connect(m_ok, SIGNAL(clicked()),
                     this, SLOT(on_okButton_clicked()));

    QObject::connect(m_cancel, SIGNAL(clicked()),
                     this, SLOT(on_cancelButton_clicked()));

    delete fileInfo;
}

void NewName::setFilesList(QList<QString> *manualList)
{
    //Set the list of files whose names will be changed
    m_list.clear();

    if(manualList)
    {
        for(QList<QString>::iterator i=manualList->begin();    i!=manualList->end();  i++)
            m_list+=*(new ParsedFile(((QString) *i)));
    }else{
        //get a list of similar files in the same directory
        qDebug() <<"pattern" <<m_sample->getPattern(); ////////////////////////////////////////
        QFileInfoList tmpList=m_directory->entryInfoList(QStringList(m_sample->getPattern()),
                                      QDir::Files | QDir::Writable,
                                      QDir::Name);

        for(QList<QFileInfo>::iterator i=tmpList.begin();    i!=tmpList.end();  i++)
            m_list+=*(new ParsedFile(((QFileInfo) *i).absoluteFilePath()));
    }
}

void NewName::on_okButton_clicked()
{
    QList<QString> *newList;
    newList=applyChangement(m_newName->text());

    if(detectDuplicates(newList))
    {
        qDebug() <<"trovati duplicati";
        for(QList<QString>::iterator i=newList->begin();    i!=newList->end();  i++)
        {
            qDebug() <<*i;
        }
        qDebug() <<"";

        QMessageBox::critical(  this, tr("Rinoma has failed -_-'"),
                                tr("Il programma non  stato in grado di capire\n"
                                "come rinominare correttamente i files. "
                                "Sorry!"),
                                QMessageBox::Close,
                                0);
        done(QDialog::Accepted);
    }else{

        Preview w(&m_list,newList,this);
        w.setWindowIcon(QIcon(":/icon.gif"));
        this->hide();

        QMessageBox msgBox;
        msgBox.setWindowIcon(QIcon(":/icon.gif"));

        switch(w.exec())
        {
        case QDialog::Accepted: //Rename the files
            rename(newList);
            msgBox.setText(tr("I files sono stati rinominati come indicato."));
            msgBox.exec();
            done(QDialog::Accepted);
            break;

        case 2: //Let the user choose which files should be renamed

            //Manually select files
            QFileDialog fileSelector;
            fileSelector.setFileMode(QFileDialog::ExistingFiles);
            fileSelector.setViewMode(QFileDialog::Detail);

            fileSelector.setWindowIcon(QIcon(":/icon.gif"));

            if(!fileSelector.exec())
            {
                done(2);
            }

            QList<QString> manualList = fileSelector.selectedFiles();

            setFilesList(&manualList);

            m_sample=new ParsedFile(QFileInfo(m_directory->absoluteFilePath(*m_sample))); //recompute
            on_okButton_clicked();
        };
    }
}

void NewName::on_cancelButton_clicked()
{
    done(QDialog::Accepted);
}

void NewName::closeEvent(QCloseEvent * event)
{
    done(QDialog::Accepted);
}

QList<QString> *NewName::applyChangement(QString newName)
{
    //Change the list of names according to the procedure
    QList<QString> *newList=new QList<QString>;
    QList<QString> *procedure=m_sample->getProcedure(newName);

    for(QList<ParsedFile>::iterator i=m_list.begin();    i!=m_list.end();  i++)
    {

        QString *changedName=((ParsedFile) *i).applyProcedure(procedure);
        //qDebug() <<(ParsedFile) *i <<" diventa " <<*changedName;
        (*newList)+=*changedName;
    }

    return newList;  
}

bool NewName::detectDuplicates(QList<QString> *newList)
{
    //Verify if there are duplicates in the list of names
    QSet<QString> set = newList->toSet();
    if(set.count()<newList->count())
        return true;
    else
        return false;
}

void NewName::rename(QList<QString> *newList)
{
    //Change the names of the files, according to the list
    QList<ParsedFile>::iterator i=m_list.begin();
    QList<QString>::iterator i2=newList->begin();

    while(i!=m_list.end())
    {
        QString oldName = (ParsedFile) *i;
        QString newName = (QString) *i2;

        if(!m_directory->rename(oldName,newName))
        {

            QMessageBox::critical(  this, tr("Rinoma has failed -_-'"),
                                    tr("Uno o pi nomi non sono stati ritenuti validi, quindi\n"
                                    "l'operazione di rinominazione dei file  stata annullata.\n"
                                    "Sorry!"),
                                    QMessageBox::Close,
                                    0);
            //Rollback
            do{
                i--;
                i2--;
                QString oldName = (ParsedFile) *i;
                QString newName = (QString) *i2;

                m_directory->rename(newName,oldName);
            }while(i!=m_list.begin());

            break;
        }

        i++;
        i2++;
    }
}
