/*#############################################################################
  # Copyright (c) 2010 Giovanni "Raptor" Pessiva <giovanni.pessiva@gmail.com> #
  # This program is free software: you can redistribute it and/or modify      #
  # it under the terms of the GNU General Public License version 3            #
  # as published by the Free Software Foundation.                             #
  #############################################################################*/
#include "parsedfile.h"
#include <QRegExp>
#include <QTextStream>
#include <QChar>
#include <QString>
#include <QDebug>
#include <QMessageBox>

ParsedFile::ParsedFile(const QFileInfo &fileinfo)
    : QString(fileinfo.fileName())
{
    m_list=parse(fileinfo.fileName());
}

QList<QString *> *ParsedFile::parse(QString name)
{
    //Divide "name" into tokens (words-numbers) and put them in a list
    QList<QString *> *list=new QList<QString *>;

    QTextStream ts(&name);
    QChar ch;

    QString tmp;
    tokenType currentType=ParsedFile::string;
    tokenType newType;
    do{
        ts >> ch;

        //Identify new char
        if(ch.isDigit())
        {
            newType=ParsedFile::number;
        }else{
            if(ch==' ' || ch=='-' || ch=='_' || ch=='.')
                newType=ParsedFile::separator;
            else
                newType=ParsedFile::string;
        }

        //save new char
        if(currentType==newType)
        {
            tmp+=ch;
        }else{
            currentType=newType;
            if(!tmp.isEmpty())
                (*list)+=new QString(tmp);
            tmp=ch;
        }
    }while(!ts.atEnd());

    /*
     //versione base (divide numeri e stringhe)
    QString tmp;
    bool text=true;
    do{
        ts >> ch;
        if(ch.isDigit())
        {
            if(!text)
                tmp+=ch;
            else{
                if(!tmp.isEmpty())
                    (*list)+=new QString(tmp);
                tmp=ch;
                text=false;
            }
        }else{
            if(text)
                tmp+=ch;
            else{
                (*list)+=new QString(tmp);
                tmp=ch;
                text=true;
            }
        }
    }while(!ts.atEnd());
    */

    if(!tmp.isEmpty())
        (*list)+=new QString(tmp);

    return list;
}

QString ParsedFile::getPattern()
{
    //Return file's pattern, used to find similar files (e.g.: photo1.jpg -> photo*.jpg)
    QString pattern;
    bool isInt;

    for(QList<QString *>::iterator i=m_list->begin();    i!=m_list->end();  i++)
    {
        ((QString *) *i)->toInt(&isInt);

        if(isInt)
            pattern+="*";
        else
            pattern+=((QString *) *i);
    }
    return pattern;
}

QList<QString> *ParsedFile::getProcedure(QString newName)
{
    //Returns the "renaming procedure" estimated from sample file
    int j;
    bool found;
    QList<QString> *proc=new QList<QString>;
    QList<QString *> *newList=parse(newName);

    //For every token, look for his previous position n, and represent it as "*n"
    for(QList<QString *>::iterator i=newList->begin();    i!=newList->end();  i++)
    {
        j=0;
        found=false;
        for(QList<QString *>::iterator i2=m_list->begin();    i2!=m_list->end() && !found;  i2++)
        {
            if(((QString) **i)==((QString) **i2))
            {
                //Token previous position found
                (*proc) <<QString("*")+QString::number(j);
                ((QString *) *i2)->append("?"); //set this string as invalid (already used)
                found=true;
            }
            j++;
        }
        if(!found) //cant find token, so it must be a new string
            (*proc) <<(QString) **i;
    }

    //remove the "?"s appended to the strings
    for(QList<QString *>::iterator i=m_list->begin();    i!=m_list->end();  i++)
        if((*i)->at((*i)->size()-1) == '?')
            (*i)->chop(1);

    //If last part of the name is unchanged, represent it as "**n"
    QListIterator<QString>      i1(*proc);
    QListIterator<QString *>    i2(*m_list);

    /*
    qDebug() <<"----proc";
    while(i1.hasNext())
    {
        QString tmp(i1.next());

        if(tmp.at(0)=='*')
        {
            tmp.remove(0,1);

            int distance=tmp.toInt();
            tmp=*(m_list->at(distance));
        }

        qDebug() <<tmp;
    }

    qDebug() <<"----m_list";
    while(i2.hasNext())
    {
        qDebug() <<(QString)  *(i2.next());
    }
    qDebug() <<"--------";
    */

    i1.toBack();
    i2.toBack();

    int j1=proc->size();
    int j2=m_list->size();
    while(i1.hasPrevious()  &&  i2.hasPrevious())
    {
        QString token1(i1.previous());

        if(token1.at(0)=='*')
        {
            token1.remove(0,1);

            int distance=token1.toInt();
            token1=*(m_list->at(distance));
        }

        QString token2(*(i2.previous()));

        //qDebug() <<"compara: '" <<token1 <<"' e '" <<token2 <<"'";

        if(token1.compare(token2))
            break;
        else{
            j1--;
            j2--;
        }
    }

    //qDebug() <<"da: " <<(QString) i1.next() <<" in poi sono uguali";
    //qDebug() <<"ovvero: " <<*(m_list->at(j2));

    proc->insert(j1,QString("**")+QString::number(j2));
    j1++;
    while(j1<proc->size())
        proc->removeAt(j1); //remove the tokens after **j2


    j1=0;
    i1.toFront();
    for(QList<QString>::iterator i=proc->begin();    i!=proc->end();  i++)
    {
        QString tmp(*i);

        if(tmp.at(0)=='*')
        {
            tmp.remove(0,1);

            int distance=tmp.toInt();
            if(distance>j2)
            {
                //replace pointers to token in the last part of the name with their value
                //this will resolve problems with names of different length
                proc->replace(j1,*(m_list->at(distance)));
            }
        }
        j1++;
    }

    qDebug() <<"contenuto di proc:";/////////////////////////////////
    for(QList<QString>::iterator i=proc->begin();    i!=proc->end();  i++)
    {
        qDebug() <<((QString) *i);

    }
    qDebug() <<"--------------" <<endl;/////////////////////////////////



    /*
      ////////////parte non completata, non funzionava, forse non aveva senso
    //Now look for unchanged consecutive tokens, and represent them as "**n"
    // e.g.: A*1B*2*3*4C*5  ->  A*1B**2C*5
    for(QList<QString>::iterator i=proc->begin();    i!=proc->end(); )
    {
        QString tmp((QString) *i);

        if(tmp.at(0)=='*')
        {
            tmp.remove(0,1);

            int distance=tmp.toInt();

            i++;

            while(i!=proc->end())
            {
                QString tmp2((QString) *i);

                if(tmp2.at(0)=='*')
                {
                    tmp2.remove(0,1);
                    if(tmp2.toInt() == distance+1)
                    {
                        //i've found 2 consecutive unchanged tokens, unify them
                        i--;
                        i=proc->insert(i,QString("**")+tmp);
                        i++;
                        i=proc->erase(i);
                        i=proc->erase(i);
                        distance++;
                    }else
                        break;
                }else
                    break;
            }
        }else
            i++;
    }

    qDebug() <<"contenuto di proc (DOPO):" <<endl;/////////////////////////////////
    for(QList<QString>::iterator i=proc->begin();    i!=proc->end();  i++)
    {
        qDebug() <<((QString) *i);

    }
    qDebug() <<"--------------" <<endl;/////////////////////////////////

    */

    return proc;
}

QString *ParsedFile::applyProcedure(QList<QString> *proc)
{
    QString *newName=new QString;
    QString tmp;

    //Apply "renaming procedure" to this file
    for(QList<QString>::iterator i=proc->begin();    i!=proc->end();  i++)
    {
        //move a token previously existing
        tmp=(QString) *i;
        if(tmp.at(0)=='*')
        { 
            tmp.remove(0,1);

            if(tmp.at(0)=='*')
            {
                tmp.remove(0,1);

                int distance=tmp.toInt();
                qDebug() <<"trovata una sequenza a partire da:" <<distance; //////////////////
                if(distance<0   ||  distance>=m_list->size())
                {
                    QMessageBox::critical(  NULL, QObject::tr("Rinoma has failed -_-'"),
                                            QObject::tr("Il programma ha incontrato dei problemi\n"
                                            "nella fase di indentificazione della procedura\n"
                                            "di rinominazione. Sorry!"),
                                            QMessageBox::Close,
                                            0);
                    exit(1);
                }

                //move all the tokens, starting with m_list[distance]
                do{
                    (*newName)+=m_list->at(distance);
                    distance++;
                }while(distance<m_list->size());


            }else{
                int distance=tmp.toInt();

                if(distance<0   ||  distance>=m_list->size())
                {
                    QMessageBox::critical(  NULL, QObject::tr("Rinoma has failed -_-'"),
                                            QObject::tr("Il programma ha incontrato dei problemi\n"
                                            "nella fase di indentificazione della procedura\n"
                                            "di rinominazione. Sorry!"),
                                            QMessageBox::Close,
                                            0);
                    exit(1);
                }

                (*newName)+=m_list->at(distance);
            }
        }else
            (*newName)+=tmp; //add this new token
    }
    return newName;
}

/*
  // incompleto e forse insensato
bool ParsedFile::lastPartMatch(QList<QString> *proc,QList<QString>::iterator i,int pos)
{
    i++;

    while(i!=proc->end()   &&  pos<m_list->size())
    {
        QString next_value=(QString) *i;

        if(next_value.at(0)=='*')
        {
            next_value.remove(0,1);

            if(next_value.at(0)=='*')
            {
                int pos2;
                for(pos2=pos;   pos2<m_list->size(); pos2++)
                {
                    if(lastPartMatch(proc,i,pos2)==true)
                        return true;
                }

                if(pos2==m_list->size())
                    return true;

                return false;

            }else{
                qDebug() <<"confronta: " <<*(m_list->at(pos)) <<" e " <<proc->at(next_value.toInt());
                if(m_list->at(pos)->compare( proc->at(next_value.toInt()) )==0)
                    continue;
                else
                    return false;
            }

        }else{
            qDebug() <<"confronta: " <<*(m_list->at(pos)) <<" e " <<next_value;
            if(m_list->at(pos)->compare(next_value)==0)
                continue;
            else
                return false;
        }

        pos++;
        i++;
    }

    return true; ///// giusto cos?
}
*/
